#ifndef _gnulib_trace_h
#define _gnulib_trace_h

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

extern int _gl_win32_winnt; /* == _WIN32_WINNT, the value Gnulib was built with */

extern int  _gl_trace_level (void);
extern void _gl_trace_color (unsigned short col);

extern char *last_component (char const *file) _GL_ATTRIBUTE_PURE;

#ifndef GL_TRACE_COLOR_START
#define GL_TRACE_COLOR_START  (8 | 2)  /* bright green */
#endif

#ifndef GL_TRACE_COLOR_ARGS
#define GL_TRACE_COLOR_ARGS   (8 | 7)  /* bright white */
#endif

#define GL_STR2(x) #x
#define GL_STR(x)  GL_STR2(x)

/*
 * The stuff in '_gl_trace_level()' should initialise itself once and
 * return the value of 'g_dbg_level'.
 */
#if defined(USE_GL_TRACE)
  #define GL_TRACE(level, fmt, ...)                                       \
          do {                                                            \
            if (_gl_trace_level() >= level) {                             \
               _gl_trace_color (GL_TRACE_COLOR_START);                    \
               printf_s ("%s(%u): ", last_component(__FILE__), __LINE__); \
               _gl_trace_color (GL_TRACE_COLOR_ARGS);                     \
               printf_s (fmt, ## __VA_ARGS__);                            \
               _gl_trace_color (0);                                       \
             }                                                            \
           } while (0)

#else
  #define GL_TRACE(level, fmt, ...)  (void)0
#endif

#ifdef __cplusplus
}
#endif

#endif  /* _gl_trace_h */

